(() => {
    if (window.hasRun) {
      return;
    }
    window.hasRun = true;
  
    console.log("Content script loaded");
  
    const HARD_TIMEOUT = 6 * 60 * 1000; // 6 minutes in milliseconds
    const SILENCE_TIMEOUT = 3000; // 3 seconds in milliseconds
	const chatServerUrl = 'https://streamchat-staging.jobsolve.ai/api/v1/speech-to-text';
  
    let mediaRecorder;
    let audioChunks = [];
    let silenceTimeout;
    let threadId = null; // Store the thread ID for continued conversation
    let isProcessing = false; // Guard to prevent multiple processing
  
    chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
      console.log("Content script received a message:", request);
      if (request.action === "startSpeechToText") {
        console.log("Starting speech-to-text process...");
        startRecording();
      } else if (request.action === "stopSpeechToText") {
        console.log("Stopping speech-to-text process...");
        stopRecording();
      }
    });
  
    async function startRecording() {
      try {
        console.log("Requesting microphone access...");
        const stream = await navigator.mediaDevices.getUserMedia({ audio: true });
        console.log("Microphone access granted.");
        mediaRecorder = new MediaRecorder(stream);
  
        mediaRecorder.ondataavailable = event => {
          audioChunks.push(event.data);
        };
  
        mediaRecorder.onstop = async () => {
          if (isProcessing) return; // Prevent multiple processing
          isProcessing = true;
  
          const audioBlob = new Blob(audioChunks, { type: 'audio/wav' });
          audioChunks = [];
          console.log("Recording stopped, audio blob created.");
          await sendAudioToAPI(audioBlob);
          chrome.runtime.sendMessage({ action: "recordingStopped" });
  
          isProcessing = false; // Reset the processing flag
        };
  
        mediaRecorder.start();
        console.log("Recording started.");
        chrome.runtime.sendMessage({ action: "updateProgress", progress: 0 });
  
        // Set a timeout for the maximum recording time
        setTimeout(() => {
          if (mediaRecorder.state === "recording") {
            mediaRecorder.stop();
          }
        }, HARD_TIMEOUT);
  
        // Stop recording after a period of silence
        detectSilence(stream, SILENCE_TIMEOUT, () => {
          if (mediaRecorder.state === "recording") {
            mediaRecorder.stop();
          }
        });
  
      } catch (error) {
        console.error("Error accessing microphone:", error);
        chrome.runtime.sendMessage({ action: "recordingStopped" });
      }
    }
  
    function stopRecording() {
      if (mediaRecorder && mediaRecorder.state === "recording") {
        mediaRecorder.stop();
      }
    }
  
    async function sendAudioToAPI(audioBlob) {
      const formData = new FormData();
      formData.append('file', audioBlob, 'speech.wav');
  
      try {
        console.log("Sending audio to Speech-to-Text API...");
        const sttResponse = await fetch(chatServerUrl, {
          method: 'POST',
          body: formData,
        });
  
        const sttResult = await sttResponse.json();
        console.log("Received response from Speech-to-Text API:", sttResult);
  
        const transcription = sttResult.transcription;
        console.log("Transcription:", transcription);
  
        const apiKey = await getStoredValue('apiKey');
        const assistantId = await getStoredValue('assistantId');
  
        if (apiKey && assistantId) {
          // If API key and assistant ID are present, send transcription to the Assistant Chat API
          await sendToAssistantChatAPI(transcription, apiKey, assistantId);
        } else {
          // Otherwise, insert the transcription directly at the cursor
          insertTextAtCursor(transcription);
        }
      } catch (error) {
        console.error('Error during API calls:', error);
      }
    }
  
    async function sendToAssistantChatAPI(transcription, apiKey, assistantId) {
      const formData = new FormData();
      formData.append('assistantId', assistantId);
      if (threadId) {
        formData.append('threadId', threadId);
      }
      formData.append('message', transcription);
  
      try {
        console.log("Sending transcription to Assistant Chat API...");
        const chatResponse = await fetch('http://localhost:30004/api/v1/assistant/chat', {
          method: 'POST',
          body: formData,
          headers: {
            'x-api-key': apiKey
          }
        });
  
        const chatResult = await chatResponse.json();
        console.log("Received response from Assistant Chat API:", chatResult);
  
        threadId = chatResult.threadId; // Store the thread ID for future requests
        chrome.storage.local.set({ threadId }); // Update threadId in storage
        insertTextAtCursor(chatResult.assistantResponse);
        chrome.runtime.sendMessage({ action: "updateThreadId", threadId }); // Notify popup to update threadId field
      } catch (error) {
        console.error('Error sending transcription to Assistant Chat API:', error);
        // Fallback to inserting the original transcription if the API call fails
        insertTextAtCursor(transcription);
      }
    }
  
    function insertTextAtCursor(text) {
        const activeElement = document.activeElement;
        console.log("Inserting text at cursor in element:", activeElement);
      
        if (activeElement.tagName === 'TEXTAREA' || activeElement.tagName === 'INPUT') {
          const start = activeElement.selectionStart;
          const end = activeElement.selectionEnd;
          activeElement.value = activeElement.value.slice(0, start) + text + activeElement.value.slice(end);
          activeElement.selectionStart = activeElement.selectionEnd = start + text.length;
          console.log(`Inserted text in TEXTAREA/INPUT: ${activeElement.tagName}`);
        } else {
          const selection = window.getSelection();
          if (selection.rangeCount > 0) {
            const range = selection.getRangeAt(0);
            const containerElement = getContainerElement(range);
      
            if (containerElement && (containerElement.isContentEditable || isReplaceableElement(containerElement))) {
              console.log("Found container element:", containerElement);
              replaceSelectedText(range, text);
            } else {
              console.warn("No valid container element found.");
              alert('Please place your cursor in an input field, textarea, or replaceable element to insert text.');
            }
          } else {
            console.warn("No selection range found.");
          }
        }
      }
      
      function getContainerElement(range) {
        let element = range.commonAncestorContainer;
        while (element && element.nodeType !== Node.ELEMENT_NODE) {
          element = element.parentElement;
        }
        return element;
      }
      
      function isReplaceableElement(element) {
        const replaceableTags = ['P', 'SPAN', 'DIV', 'H1', 'H2', 'H3', 'LI', 'TD'];
        return replaceableTags.includes(element.tagName);
      }
      
      function replaceSelectedText(range, text) {
        range.deleteContents();
        const textNode = document.createTextNode(text);
        range.insertNode(textNode);
        range.setStartAfter(textNode);
        range.setEndAfter(textNode);
        const selection = window.getSelection();
        selection.removeAllRanges();
        selection.addRange(range);
        console.log("Replaced selected text with new text.");
      }
      
      function simulateTyping(element, text) {
        const selection = window.getSelection();
        const range = selection.getRangeAt(0);
        console.log("Initial range:", range);
      
        for (const char of text) {
          const event = new KeyboardEvent('keypress', {
            bubbles: true,
            cancelable: true,
            key: char,
            charCode: char.charCodeAt(0)
          });
          element.dispatchEvent(event);
      
          const inputEvent = new InputEvent('input', {
            bubbles: true,
            cancelable: true,
            inputType: 'insertText',
            data: char
          });
          element.dispatchEvent(inputEvent);
      
          const textNode = document.createTextNode(char);
          range.insertNode(textNode);
          console.log(`Inserted character '${char}' into the contenteditable element`);
      
          // Move the range after the newly inserted character
          range.setStartAfter(textNode);
          range.setEndAfter(textNode);
          selection.removeAllRanges();
          selection.addRange(range);
          console.log("Updated range after inserting character:", char);
        }
      }
      
      
      
      
      
      
      
      
  
    // Function to detect silence in the audio stream
    function detectSilence(stream, silenceDelay, onSilence) {
      const audioContext = new (window.AudioContext || window.webkitAudioContext)();
      const analyser = audioContext.createAnalyser();
      const microphone = audioContext.createMediaStreamSource(stream);
      const scriptProcessor = audioContext.createScriptProcessor(256, 1, 1);
  
      analyser.smoothingTimeConstant = 0.8;
      analyser.fftSize = 1024;
  
      microphone.connect(analyser);
      analyser.connect(scriptProcessor);
      scriptProcessor.connect(audioContext.destination);
      scriptProcessor.onaudioprocess = function() {
        const array = new Uint8Array(analyser.frequencyBinCount);
        analyser.getByteFrequencyData(array);
        const values = array.reduce((a, b) => a + b, 0) / array.length;
  
        if (values < 10) {
          if (!silenceTimeout) {
            silenceTimeout = setTimeout(() => {
              onSilence();
            }, silenceDelay);
          }
        } else {
          clearTimeout(silenceTimeout);
          silenceTimeout = null;
        }
      };
    }
  
    // Helper function to get a stored value from chrome.storage.local
    async function getStoredValue(key) {
      return new Promise((resolve, reject) => {
        chrome.storage.local.get([key], (result) => {
          if (chrome.runtime.lastError) {
            return reject(chrome.runtime.lastError);
          }
          resolve(result[key]);
        });
      });
    }
  })();
  